@@ -32,7 +32,7 @@
 /* bindings and mouse buttons */
 /* Fvwm needs at least 3 buttons. X currently supports up to 5 buttons, so you
  * can use 3, 4 or 5 here. */
-#define NUMBER_OF_MOUSE_BUTTONS            3
+#define NUMBER_OF_MOUSE_BUTTONS            5
 #define DEFAULT_BUTTONS_TO_GRAB            ((1 << NUMBER_OF_MOUSE_BUTTONS) - 1)
 #define DEFAULT_ALL_BUTTONS_MASK           \
 ((Button1Mask * (NUMBER_OF_MOUSE_BUTTONS >= 1)) | \
@@ -70,6 +70,12 @@
 /* this value is used in a bugfix */
 #define WINDOW_FREAKED_OUT_HEIGHT      65500
 
+/* window placement (cleverplacement) */
+#define PLACEMENT_AVOID_STICKY 1
+#define PLACEMENT_AVOID_ONTOP 5
+#define PLACEMENT_AVOID_ICON 10   /*  Try hard no to place windows over icons */
+/*#define PLACEMENT_AVOID_ICON 0*//*  Ignore Icons.  Place windows over them  */
+
 /* movement */
 #define DEFAULT_OPAQUE_MOVE_SIZE           5
 #define DEFAULT_SNAP_ATTRACTION           -1
